// Modified by Eric Wong, Oct. 8th, 2008
// VideoInPicture@gmail.com
// Added the ability to consume physical memory in addition to virtual memory.

using System;
using System.Collections;
using System.Runtime.InteropServices;
using System.Text;

namespace EatMem
{
	class Class1
	{
		[DllImport("kernel32.dll")]
		extern static IntPtr GlobalAlloc(uint uFlags, uint dwBytes);
		// HGLOBAL GlobalAlloc(UINT uFlags, SIZE_T dwBytes);

		[DllImport("kernel32.dll")]
		extern static IntPtr GlobalFree(IntPtr hMem);
		// HGLOBAL GlobalFree(HGLOBAL hMem);

		//[DllImport("kernel32.dll")]
		//extern static  IntPtr VirtualAlloc(IntPtr lpAddress, uint dwSize, uint flAllocationType, uint flProtect);
		// LPVOID VirtualAlloc(LPVOID lpAddress, SIZE_T dwSize, DWORD flAllocationType, DWORD flProtect);

		//[DllImport("kernel32.dll")]
		//extern static int VirtualFree(IntPtr lpAddress, uint dwSize, uint dwFreeType);
		// BOOL VirtualFree(LPVOID lpAddress, SIZE_T dwSize, DWORD dwFreeType);

        [DllImport("kernel32.dll")]
        extern static IntPtr GetCurrentProcess();

        [DllImport("psapi.dll")]
        extern static int GetProcessMemoryInfo(IntPtr hProcess, out
        PROCESS_MEMORY_COUNTERS counters, int size);

        //[StructLayout(LayoutKind.Sequential, Size = 40)]
        struct PROCESS_MEMORY_COUNTERS
        {
            public int cb;
            public int PageFaultCount;
            public int PeakWorkingSetSize;
            public int WorkingSetSize;
            public int QuotaPeakPagedPoolUsage;
            public int QuotaPagedPoolUsage;
            public int QuotaPeakNonPagedPoolUsage;
            public int QuotaNonPagedPoolUsage;
            public int PagefileUsage;
            public int PeakPagefileUsage;
        }

		[DllImport("kernel32.dll")]
		extern static void GlobalMemoryStatus(ref MEMORYSTATUS lpBuffer);
    
		//[DllImport("kernel32.dll")]
		//extern static void GetSystemInfo(ref SYSTEM_INFO lpSystemInfo);

		[DllImport("kernel32.dll")]
		extern static IntPtr GetStdHandle (int nStdHandle);

		[DllImport("kernel32.dll")]
		extern static bool SetConsoleTextAttribute(IntPtr hConsoleOutput, int wAttributes);

		const int BLACK = 0x0000;
		const int BLUE = 0x0001 | 0x0008;
		const int GREEN = 0x0002 | 0x0008;
		const int CYAN = 0x0003 | 0x0008;
		const int RED = 0x0004 | 0x0008;
		const int MAGENTA = 0x0005 | 0x0008;
		const int YELLOW = 0x0006 | 0x0008;
		const int WHITE = 0x0007 | 0x0008;

		const int BGWHITE = 0x0070 | 0x0080;
		const int BGBLACK = 0x0000 | 0x0000;
		const int BGGRAY = 0x0070;

		static MEMORYSTATUS memStatusBuffer = new MEMORYSTATUS();
		//static SYSTEM_INFO sysInfoBuffer = new SYSTEM_INFO();

		[STAThread]
		static void Main(string[] args)
		{
			try
			{
				//GetSystemInfo(ref sysInfoBuffer);
				//uint pageSize = sysInfoBuffer.dwPageSize;
				//Console.WriteLine("page size is " + pageSize); // 4KB = 4096 bytes typical
				//Console.WriteLine();

				const int STD_OUTPUT_HANDLE = -11;
				IntPtr hStdOut = GetStdHandle(STD_OUTPUT_HANDLE);

				const uint basicUnit = 1024; // 1 KB
       
				DateTime startTime = DateTime.Now;
   
				if (args.Length != 1 || !args[0].StartsWith("-d"))
				{
					SetConsoleTextAttribute(hStdOut, WHITE | BGBLACK);
					Console.WriteLine("\nExample usage: eatmem -d100");
					Console.WriteLine("Randomly eats between 50% and 100% of physical & virtual memory for 100 minutes");
					return;
				}

				int duration = int.Parse(args[0].Substring(2, args[0].Length-2));
				TimeSpan ts = new TimeSpan(0, duration, 0); // hr, min, sec
				DateTime stopTime = startTime + ts;
				DateTime currentTime = DateTime.Now;

				SetConsoleTextAttribute(hStdOut, WHITE | BGBLACK);
				Console.WriteLine("\nBegin EatMem for " + duration + " minutes");
				Console.WriteLine("Consuming random 50%-100% of physical & virtual memory");
				Console.WriteLine("Launch your test harness while EatMem stressor runs. . .\n");

				Random r = new Random();

				// get initial memory status data if necessary
				//GlobalMemoryStatus(ref memStatusBuffer);
				//uint lengthInKB = memStatusBuffer.dwLength / (uint)basicUnit;
				//uint memoryLoadInKB = memStatusBuffer.dwMemoryLoad / (uint)basicUnit;
				//uint totalPhysicalInKB = memStatusBuffer.dwTotalPhys / (uint)basicUnit;
				//uint availablePhysicalInKB = memStatusBuffer.dwAvailPhys / (uint)basicUnit;
				//uint toatlPageFileInKB = memStatusBuffer.dwTotalPageFile / (uint)basicUnit;
				//uint availablePageFileInKB = memStatusBuffer.dwAvailPageFile / (uint)basicUnit;
				//uint totalVirtualInKB = memStatusBuffer.dwTotalVirtual / (uint)basicUnit;
				//uint availableVirtualInKB = memStatusBuffer.dwAvailVirtual / (uint)basicUnit;

				//print header
				SetConsoleTextAttribute(hStdOut, YELLOW | BGBLACK);
				Console.WriteLine("           Available    Available      Current");
				Console.WriteLine("            Physical      Virtual       Memory");
				Console.WriteLine(" Time         Memory       Memory   Allocation   Pct. Avail. PM+VM Allocated");
				Console.WriteLine("----------------------------------------------------------------------------");
				//Console.WriteLine("         1         2         3         4         5         6         7   ");
				//Console.WriteLine("1234567890123456789012345678901234567890123456789012345678901234567890123");

				while (currentTime  < stopTime) // main loop
				{
					for (int cycle = 0; cycle < 8; ++cycle) // arbitray 8 cycles for readability
					{
						// get the current time
						currentTime = DateTime.Now;
						TimeSpan elapsedTime = currentTime - startTime;
						string time; // for display later

						// compute percent mem to allocate
						double randomPct = (r.NextDouble() * (1.0 - 0.5)) + 0.5; // [0.5 and 1.0]
						if (randomPct < 0.5 || randomPct > 1.0)
							throw new Exception("Bad random pct");

						// need to update memory status to compute bytes to eat
						GlobalMemoryStatus(ref memStatusBuffer);
						uint numBytesToEatInBytes = (uint)(randomPct * memStatusBuffer.dwAvailVirtual); // feed to alloc()
						uint numBytesToEatInKB = numBytesToEatInBytes / basicUnit; // for display purposes only


                        uint numBytesToEatInBytesPhysical = (uint)(randomPct * memStatusBuffer.dwAvailPhys); // feed to alloc()
                        uint numBytesToEatInKBPhysical = numBytesToEatInBytesPhysical / basicUnit; // for display purposes only


						//Console.WriteLine("availableVirtualInKB = " + availableVirtualInKB);
						//Console.WriteLine("randomPct = " + randomPct);
						//Console.WriteLine("numBytesToEatInBytes = " + numBytesToEatInBytes);
						//Console.WriteLine("numBytesToEatInKB = " + numBytesToEatInKB);
						//Console.WriteLine("Now callingb GlobalAlloc() . . . ");

						// allocate memory
						// GlobalAlloc
						uint GPTR = 0x0040; // Combines GMEM_FIXED and GMEM_ZEROINIT
						IntPtr p =  GlobalAlloc(GPTR, numBytesToEatInBytes);

                        
                        // Eat up the physical memory
                        //Console.WriteLine("Trying to Eat # Physical Bytes: " + numBytesToEatInBytesPhysical);
                        ArrayList physicalMemoryEater = new ArrayList();
                        PROCESS_MEMORY_COUNTERS memoryCounter = new PROCESS_MEMORY_COUNTERS();
                        GetProcessMemoryInfo(GetCurrentProcess(), out memoryCounter, Marshal.SizeOf(memoryCounter));
                        while (memoryCounter.WorkingSetSize < numBytesToEatInBytesPhysical)
                        {
                            System.Drawing.Bitmap physicalMemoryBitmap = new System.Drawing.Bitmap(1000, 1000, System.Drawing.Imaging.PixelFormat.Format8bppIndexed);
                            physicalMemoryEater.Add(physicalMemoryBitmap);
                            GetProcessMemoryInfo(GetCurrentProcess(), out memoryCounter, Marshal.SizeOf(memoryCounter));
                        }

                        //if (p == IntPtr.Zero)
                        //{
                        //    time = elapsedTime.Hours.ToString("00") + ":" + elapsedTime.Minutes.ToString("00") + ":" + elapsedTime.Seconds.ToString("00");
                        //    SetConsoleTextAttribute(hStdOut, CYAN | BGBLACK);
                        //    Console.Write("{0,8}", time);
                        //    SetConsoleTextAttribute(hStdOut, RED | BGBLACK);
                        //    Console.WriteLine("   GlobalAlloc() failed.  Trying again");
                        //    --cycle;
                        //    continue;
                        //}

						// VirtualAlloc
						//uint MEM_RESERVE = 0x1000;
						//uint MEM_COMMIT = 0x2000;
						//uint PAGE_READWRITE = 0x04;
						//IntPtr p = VirtualAlloc(IntPtr.Zero, numBytesToEatInBytes, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE);
						//if (p == IntPtr.Zero)
						//{
						//  time = elapsedTime.Hours.ToString("00") + ":" + elapsedTime.Minutes.ToString("00") + ":" + elapsedTime.Seconds.ToString("00");
						//  SetConsoleTextAttribute(hStdOut, CYAN | BGBLACK);
						//  Console.Write("{0,8}", time);
						//  SetConsoleTextAttribute(hStdOut, RED | BGBLACK);
						//  Console.WriteLine("   VirtualAlloc() failed.  Trying again");
						//  --cycle;
						//  continue;
						//}

						// alloc succeeded so
						// get current memory info after alloc(), before free() for display
						GlobalMemoryStatus(ref memStatusBuffer);
						//uint lengthInKB = memStatusBuffer.dwLength / (uint)basicUnit;
						//uint memoryLoadInKB = memStatusBuffer.dwMemoryLoad / (uint)basicUnit;
						//uint totalPhysicalInKB = memStatusBuffer.dwTotalPhys / (uint)basicUnit;
						uint availablePhysicalInKB = memStatusBuffer.dwAvailPhys / (uint)basicUnit;
						//uint toatlPageFileInKB = memStatusBuffer.dwTotalPageFile / (uint)basicUnit;
						//uint availablePageFileInKB = memStatusBuffer.dwAvailPageFile / (uint)basicUnit;
						//uint totalVirtualInKB = memStatusBuffer.dwTotalVirtual / (uint)basicUnit;
						uint availableVirtualInKB = memStatusBuffer.dwAvailVirtual / (uint)basicUnit;
						numBytesToEatInBytes = (uint)(randomPct * memStatusBuffer.dwAvailVirtual); // feed to alloc()
						numBytesToEatInKB = numBytesToEatInBytes / basicUnit; // for display purposes only

						// print time
						time = elapsedTime.Hours.ToString("00") + ":" + elapsedTime.Minutes.ToString("00") + ":" + elapsedTime.Seconds.ToString("00");
						SetConsoleTextAttribute(hStdOut, CYAN | BGBLACK);
						Console.Write("{0,8}", time);

						// print the memory numbers
						SetConsoleTextAttribute(hStdOut, GREEN | BGBLACK);
                        Console.Write("{0,12} {1,12} {2,12}", availablePhysicalInKB + " KB", availableVirtualInKB + " KB", numBytesToEatInKB + numBytesToEatInKBPhysical + " KB");
            
						// print the current alooc percent number
						SetConsoleTextAttribute(hStdOut, MAGENTA | BGBLACK);
						Console.Write("   " +  (int)(randomPct*100.0));
						SetConsoleTextAttribute(hStdOut, GREEN | BGBLACK);
						Console.Write("|" );

						// do the bars
						uint totalNumBars = 20;
						uint eachBarIs = (availableVirtualInKB + availablePhysicalInKB) / totalNumBars;
                        uint barsToPrint = (numBytesToEatInKB + numBytesToEatInKBPhysical) / eachBarIs;
                        if (barsToPrint < 0)
                            barsToPrint = 0;
                        
                        if (barsToPrint > 20)
                            barsToPrint = 20;

						string bars = new string('_', (int)barsToPrint);
						string spaces = new string('_', 20-(int)barsToPrint);

						SetConsoleTextAttribute(hStdOut, BLACK | BGWHITE);
						Console.Write(bars);

						SetConsoleTextAttribute(hStdOut, WHITE | BGBLACK);
						Console.Write(spaces);

						SetConsoleTextAttribute(hStdOut, GREEN | BGBLACK);
						Console.WriteLine("|");

						System.Threading.Thread.Sleep(5000); // give app under stress time

						// free memory

						// GlobalFree 
						IntPtr freeResult = GlobalFree(p);
                        physicalMemoryEater = null;
                        GC.Collect();
						if (freeResult != IntPtr.Zero)
							throw new Exception("GlobalFree() failed");

						// VirtualFree
						//uint MEM_RELEASE = 0x8000;
						//int freeResult = VirtualFree(p, 0, MEM_RELEASE);
						//if (freeResult == 0)
						//  throw new Exception("VirtualFree() failed !");

					} // for loop
          
					SetConsoleTextAttribute(hStdOut, YELLOW | BGGRAY);
					Console.WriteLine(" Time       Physical      Virtual   Allocation");
				} // while

				SetConsoleTextAttribute(hStdOut, WHITE | BGBLACK);
				Console.WriteLine("\nDone");
				//Console.ReadLine();
			}
			catch(Exception ex)
			{
				Console.WriteLine("Fatal error: " + ex.Message);
				Console.ReadLine();
			}

		} // Main()

	} // class

	public struct MEMORYSTATUS
	{
		public  uint dwLength;
		public  uint dwMemoryLoad;
		public  uint dwTotalPhys;
		public  uint dwAvailPhys;
		public  uint dwTotalPageFile;
		public  uint dwAvailPageFile;
		public  uint dwTotalVirtual;
		public  uint dwAvailVirtual;
	}
	//  public struct SYSTEM_INFO 
	//  {
	//    public  uint  dwOemId;
	//    public  uint  dwPageSize;
	//    public  uint  lpMinimumApplicationAddress;
	//    public	uint  lpMaximumApplicationAddress;
	//    public  uint  dwActiveProcessorMask;
	//    public  uint  dwNumberOfProcessors;
	//    public  uint  dwProcessorType;
	//    public  uint  dwAllocationGranularity;
	//    public  uint  dwProcessorLevel;
	//    public  uint  dwProcessorRevision;		
	//  }


} // ns
